@@ -1,6 +1,6 @@
 __all__ = [
-    "NotNull",
-    "Skip",
+    "Undefined",
+    "UndefinedType",
     "Unsupported",
     "ValidationError",
     "alias",
@@ -18,8 +18,10 @@ __all__ = [
     "schema_ref",
     "serialization",
     "serialize",
+    "serialized",
     "serializer",
     "settings",
+    "skip",
     "validation",
     "validator",
 ]
@@ -28,12 +30,12 @@ __all__ = [
 from . import (
     conversions,
     dataclasses,
-    deserialization,
     fields,
     json_schema,
     metadata,
     serialization,
     settings,
+    skip,
     validation,
 )
 from .aliases import alias
@@ -43,19 +45,32 @@ from .deserialization import deserialize
 from .json_schema.refs import schema_ref
 from .json_schema.schema import schema
 from .metadata import properties
-from .serialization import serialize
-from .skip import NotNull, Skip
-from .validation import (
-    ValidationError,
-    validator,
-)
+from .serialization import serialize, serialized
+from .utils import Undefined, UndefinedType
+from .validation import ValidationError, validator
 from .visitor import Unsupported
 
+try:
+    from . import graphql  # noqa: F401
+
+    __all__.append("graphql")
+except ImportError:
+    pass
+
+
+def __getattr__(name):
+    if name == "graphql":
+        raise AttributeError(
+            "GraphQL feature requires graphql-core library\n"
+            "Run `pip install apischema[graphql]` to install it"
+        )
+    raise AttributeError(f"module {__name__!r} has no attribute {name!r}")
+
 
 def default_conversions():
     """Handle standard library + internal types"""
     from typing import Sequence
-    from . import std_types  # noqa F401
+    from . import std_types  # noqa: F401
     from .validation.errors import LocalizedError
 
     deserializer(ValidationError.deserialize, Sequence[LocalizedError], ValidationError)
