@@ -1,10 +1,13 @@
 __all__ = [
-    "Deserialization",
     "Conversions",
+    "Deserialization",
     "Serialization",
+    "as_str",
+    "dataclass_model",
     "deserializer",
     "extra_deserializer",
     "extra_serializer",
+    "identity",
     "inherited_deserializer",
     "raw_deserializer",
     "reset_deserializers",
@@ -13,6 +16,7 @@ __all__ = [
 ]
 
 from .converters import (
+    as_str,
     deserializer,
     extra_deserializer,
     extra_serializer,
@@ -22,5 +26,6 @@ from .converters import (
     serializer,
 )
 from .raw import raw_deserializer
-from .utils import Conversions
+from .utils import Conversions, identity
 from .visitor import Deserialization, Serialization
+from .dataclass_models import dataclass_model
