@@ -2,10 +2,17 @@ from collections import defaultdict
 from contextlib import suppress
 from functools import wraps
 from inspect import signature
+from types import new_class
 from typing import (
+    Any,
     Callable,
     Dict,
+    Mapping,
+    NewType,
     Optional,
+    Pattern,
+    Sequence,
+    TYPE_CHECKING,
     Tuple,
     Type,
     TypeVar,
@@ -19,9 +26,13 @@ from apischema.conversions.utils import (
     ConverterWithConversions,
     check_converter,
     check_convertible,
-    use_origin_type_vars,
+    handle_generic_conversions,
 )
 from apischema.types import AnyType, OrderedDict
+from apischema.utils import Undefined, to_camel_case
+
+if TYPE_CHECKING:
+    from apischema.json_schema.annotations import Deprecated
 
 _deserializers: Dict[AnyType, Dict[AnyType, ConverterWithConversions]] = defaultdict(
     OrderedDict
@@ -141,7 +152,7 @@ def _deserializer(
     conversions: Optional[Conversions],
     extra: bool,
 ) -> Func:
-    ret, param = use_origin_type_vars(ret, param)
+    ret, param = handle_generic_conversions(ret, param)
     check_convertible(ret)
     if param == ret:
         raise ValueError("Use self_deserializer")
@@ -162,11 +173,11 @@ def _serializer(
     conversions: Optional[Conversions],
     extra: bool,
 ) -> Func:
-    param, ret = use_origin_type_vars(param, ret)
+    param, ret = handle_generic_conversions(param, ret)
     check_convertible(param)
     if param == ret:
         if conversions:
-            raise ValueError("Self-conversion cannot have conversions parameter")
+            raise ValueError("Self-conversions cannot have conversions parameter")
         if not extra:
             _serializers.pop(param, ...)
     else:
@@ -184,7 +195,7 @@ Cls = TypeVar("Cls", bound=Type)
 
 
 def self_deserializer(cls: Cls) -> Cls:
-    _deserializer(lambda x: x, type(cls.__name__, (cls,), {}), cls, None, extra=False)
+    _deserializer(lambda x: x, new_class(cls.__name__, (cls,)), cls, None, extra=False)
     return cls
 
 
@@ -238,3 +249,33 @@ def inherited_deserializer(
     if not isinstance(func, classmethod):
         raise TypeError("inherited_deserializer must be called on classmethod")
     return InheritedDeserializer(func, conversions, extra)
+
+
+def as_str(
+    cls: Cls,
+    *,
+    title: Optional[str] = None,
+    description: Optional[str] = None,
+    default: Any = Undefined,
+    examples: Optional[Sequence[Any]] = None,
+    deprecated: "Deprecated" = False,
+    format: Optional[str] = None,
+    min_len: Optional[int] = None,
+    max_len: Optional[int] = None,
+    pattern: Optional[Union[str, Pattern]] = None,
+    extra: Mapping[str, Any] = None,
+    override: bool = False,
+) -> Cls:
+    ...
+
+
+@wraps(as_str)
+def _as_str(cls: Type, **kwargs):
+    from apischema import schema
+
+    str_type = schema(**kwargs)(NewType(to_camel_case(cls.__name__), str))
+    deserializer(cls, str_type, cls)
+    serializer(str, cls, str_type)
+
+
+globals()[as_str.__name__] = _as_str
