@@ -1,4 +1,5 @@
 from contextlib import contextmanager
+from dataclasses import Field, replace
 from enum import Enum
 from functools import wraps
 from itertools import chain
@@ -19,19 +20,26 @@ from typing import (  # type: ignore
     cast,
 )
 
-from dataclasses import Field
-
 from apischema import settings
+from apischema.aliases import Aliaser
 from apischema.conversions.utils import Conversions
-from apischema.conversions.visitor import Conv
+from apischema.conversions.visitor import (
+    Conv,
+    ConversionsVisitor,
+    DeserializationVisitor,
+    SerializationVisitor,
+)
 from apischema.dataclass_utils import (
     get_alias,
     get_default,
+    get_field_conversion,
+    get_fields,
+    get_requirements,
     has_default,
     is_dataclass,
     is_required,
 )
-from apischema.dataclasses import replace
+from apischema.dependent_required import DependentRequired
 from apischema.json_schema.constraints import (
     ArrayConstraints,
     Constraints,
@@ -40,11 +48,6 @@ from apischema.json_schema.constraints import (
     StringConstraints,
 )
 from apischema.json_schema.generation.refs import Refs, RefsExtractor
-from apischema.json_schema.generation.visitor import (
-    DeserializationSchemaVisitor,
-    SchemaVisitor,
-    SerializationSchemaVisitor,
-)
 from apischema.json_schema.patterns import infer_pattern
 from apischema.json_schema.refs import get_ref, schema_ref
 from apischema.json_schema.schema import Schema, get_schema, merge_schema
@@ -56,9 +59,11 @@ from apischema.metadata.keys import (
     SCHEMA_METADATA,
     check_metadata,
 )
-from apischema.serialization import serialize
-from apischema.types import AnyType
-from apischema.utils import is_hashable
+from apischema.serialization import get_serialized_resolvers, serialize
+from apischema.skip import filter_skipped
+from apischema.types import AnyType, OrderedDict
+from apischema.typing import get_origin
+from apischema.utils import Operation, is_hashable
 
 constraint_by_type = {
     int: NumberConstraints,
@@ -73,40 +78,41 @@ Method = TypeVar("Method", bound=Callable[..., JsonSchema])
 def with_schema(method: Method) -> Method:
     @wraps(method)
     def wrapper(self: "SchemaBuilder", *args, **kwargs):
-        if self.schema is None:
+        if self._schema is None:
             return method(self, *args, **kwargs)
-        elif self.schema.override:
-            return JsonSchema(**self.schema.as_dict())
+        elif self._schema.override:
+            return JsonSchema(**self._schema.as_dict())
         else:
-            return JsonSchema(method(self, *args, **kwargs), **self.schema.as_dict())
+            return JsonSchema(method(self, *args, **kwargs), **self._schema.as_dict())
 
     return cast(Method, wrapper)
 
 
-class SchemaBuilder(SchemaVisitor[Conv, JsonSchema]):
+class SchemaBuilder(ConversionsVisitor[Conv, JsonSchema]):
     def __init__(
         self,
-        conversions: Optional[Conversions],
+        aliaser: Aliaser,
         ref_factory: RefFactory,
         refs: Collection[str],
         ignore_first_ref: bool,
     ):
-        super().__init__(conversions)
+        super().__init__()
         self.ref_factory = ref_factory
         self.refs = refs
-        self.ignore_first_ref = ignore_first_ref
-        self.schema: Optional[Schema] = None
+        self.aliaser = aliaser
+        self._ignore_first_ref = ignore_first_ref
+        self._schema: Optional[Schema] = None
 
     def _check_constraints(self, expected: Type[Constraints]):
-        if self.schema is not None and self.schema.constraints is not None:
-            if not isinstance(self.schema.constraints, expected):
+        if self._schema is not None and self._schema.constraints is not None:
+            if not isinstance(self._schema.constraints, expected):
                 raise TypeError(
                     f"Bad constraints: expected {expected.__name__}"
-                    f" found {type(self.schema.constraints).__name__}"
+                    f" found {type(self._schema.constraints).__name__}"
                 )
 
     def _merge_schema(self, schema: Optional[Schema]):
-        self.schema = merge_schema(schema, self.schema)
+        self._schema = merge_schema(schema, self._schema)
 
     @with_schema
     def _ref_schema(self, ref: str) -> JsonSchema:
@@ -117,8 +123,8 @@ class SchemaBuilder(SchemaVisitor[Conv, JsonSchema]):
             if isinstance(annotation, schema_ref):
                 annotation.check_type(cls)
                 if annotation.ref in self.refs:
-                    if self.ignore_first_ref:
-                        self.ignore_first_ref = False
+                    if self._ignore_first_ref:
+                        self._ignore_first_ref = False
                     else:
                         assert isinstance(annotation.ref, str)
                         return self._ref_schema(annotation.ref)
@@ -127,7 +133,7 @@ class SchemaBuilder(SchemaVisitor[Conv, JsonSchema]):
                     raise ValueError("Annotated schema_ref can only be str")
             if isinstance(annotation, Schema):
                 self._merge_schema(annotation)
-        return self.visit_with_schema(cls, self.schema)
+        return self.visit_with_schema(cls, self._schema)
 
     @with_schema
     def any(self) -> JsonSchema:
@@ -142,26 +148,29 @@ class SchemaBuilder(SchemaVisitor[Conv, JsonSchema]):
             uniqueItems=issubclass(cls, AbstractSet),
         )
 
-    def _visit_field_(self, field: Field, field_type: AnyType):
+    def visit_field(self, field: Field, field_type: AnyType):
+        field_type, conversions, _ = get_field_conversion(
+            field, field_type, self.operation
+        )
+        schema: Optional[Schema] = None
         if SCHEMA_METADATA in field.metadata:
-            schema: Schema = field.metadata[SCHEMA_METADATA]
+            schema = cast(Schema, field.metadata[SCHEMA_METADATA])
             if schema.annotations is not None and schema.annotations is ...:
                 if not has_default(field):
                     raise TypeError("Invalid ... without field default")
                 try:
-                    default = serialize(get_default(field))
+                    default = serialize(get_default(field), conversions=conversions)
                 except Exception:
                     pass
                 else:
                     annotations = replace(schema.annotations, default=default)
                     schema = replace(schema, annotations=annotations)
+        with self._replace_conversions(conversions):
             return self.visit_with_schema(field_type, schema)
-        else:
-            return self.visit(field_type)
 
     def _properties_schema(self, field: Field, field_type: AnyType) -> JsonSchema:
         with self._without_ref():
-            props_schema = self._visit_field(field, field_type)
+            props_schema = self.visit_field(field, field_type)
         if not props_schema.get("type") == JsonType.OBJECT:
             raise TypeError("properties field must have an 'object' type")
         if "patternProperties" in props_schema:
@@ -179,6 +188,21 @@ class SchemaBuilder(SchemaVisitor[Conv, JsonSchema]):
                 pass
         return JsonSchema()
 
+    def _check_merged_schema(self, cls: Type, field: Field, field_type: AnyType):
+        ignore_first_ref = self._ignore_first_ref
+        self._ignore_first_ref = True
+        try:
+            if self.visit_field(field, field_type).get("type") not in {
+                JsonType.OBJECT,
+                "object",
+            }:
+                raise TypeError(
+                    f"Merged field {cls.__name__}.{field.name}"
+                    f" must have an object type"
+                )
+        finally:
+            self._ignore_first_ref = ignore_first_ref
+
     @with_schema
     def dataclass(
         self,
@@ -194,11 +218,12 @@ class SchemaBuilder(SchemaVisitor[Conv, JsonSchema]):
         merged_schemas = []
         pattern_properties = {}
         additional_properties: Union[bool, JsonSchema] = settings.additional_properties
-        for field in self._dataclass_fields(cls, fields, init_vars):
+        for field in get_fields(fields, init_vars, self.operation):
             metadata = check_metadata(field)
             field_type = types[field.name]
             if MERGED_METADATA in metadata:
-                merged_schemas.append(self._visit_field(field, field_type))
+                self._check_merged_schema(cls, field, field_type)
+                merged_schemas.append(self.visit_field(field, field_type))
             elif PROPERTIES_METADATA in metadata:
                 pattern = metadata[PROPERTIES_METADATA]
                 properties_schema = self._properties_schema(field, field_type)
@@ -209,21 +234,38 @@ class SchemaBuilder(SchemaVisitor[Conv, JsonSchema]):
                 else:
                     pattern_properties[pattern] = properties_schema
             else:
-                alias = get_alias(field)
+                alias = self.aliaser(get_alias(field))
                 properties[alias] = json_schema(
                     readOnly=not field.init,
                     writeOnly=field in init_vars,
-                    **self._visit_field(field, field_type),
+                    **self.visit_field(field, field_type),
                 )
                 if is_required(field):
                     required.append(alias)
+        if self.operation == Operation.SERIALIZATION:
+            for name, resolver in get_serialized_resolvers(cls).items():
+                with self._replace_conversions(resolver.conversions):
+                    properties[self.aliaser(name)] = json_schema(
+                        readOnly=True,
+                        **self.visit_with_schema(resolver.return_type, resolver.schema),
+                    )
+        dependent_required = {
+            self.aliaser(get_alias(field)): sorted(
+                self.aliaser(get_alias(req)) for req in required_by
+            )
+            for field, required_by in get_requirements(
+                cls, DependentRequired.requiring, self.operation
+            ).items()
+        }
         result = json_schema(
             type=JsonType.OBJECT,
             properties=properties,
             required=required,
             additionalProperties=additional_properties,
             patternProperties=pattern_properties,
-            dependentRequired=self._dependent_required(cls),
+            dependentRequired=OrderedDict(
+                (f, dependent_required[f]) for f in sorted(dependent_required)
+            ),
         )
         if merged_schemas:
             result = json_schema(
@@ -238,6 +280,14 @@ class SchemaBuilder(SchemaVisitor[Conv, JsonSchema]):
             raise TypeError("Empty enum")
         return self.literal(list(cls))
 
+    def generic(self, cls: AnyType) -> JsonSchema:
+        origin = get_origin(cls)
+        if is_hashable(origin) and self.is_extra_conversions(origin):
+            self._schema = None
+        else:
+            self._merge_schema(get_schema(origin))
+        return super().generic(cls)
+
     @with_schema
     def literal(self, values: Sequence[Any]) -> JsonSchema:
         if not values:
@@ -285,13 +335,15 @@ class SchemaBuilder(SchemaVisitor[Conv, JsonSchema]):
         self._check_constraints(ObjectConstraints)
         return json_schema(
             type=JsonType.OBJECT,
-            properties={key: self.visit(key) for key, cls in types.items()},
+            properties={
+                self.aliaser(key): self.visit(key) for key, cls in types.items()
+            },
             required=sorted(types.keys() - defaults.keys()),
             additionalProperties=settings.additional_properties,
         )
 
     def new_type(self, cls: Type, super_type: AnyType) -> JsonSchema:
-        return self.visit_with_schema(super_type, self.schema)
+        return self.visit_with_schema(super_type, self._schema)
 
     @with_schema
     def primitive(self, cls: Type) -> JsonSchema:
@@ -305,11 +357,11 @@ class SchemaBuilder(SchemaVisitor[Conv, JsonSchema]):
     @with_schema
     def tuple(self, types: Sequence[AnyType]) -> JsonSchema:
         self._check_constraints(ArrayConstraints)
-        if self.schema is not None and self.schema.constraints is not None:
-            assert isinstance(self.schema.constraints, ArrayConstraints)
+        if self._schema is not None and self._schema.constraints is not None:
+            assert isinstance(self._schema.constraints, ArrayConstraints)
             if (
-                self.schema.constraints.max_items is not None
-                or self.schema.constraints.min_items is not None
+                self._schema.constraints.max_items is not None
+                or self._schema.constraints.min_items is not None
             ):
                 raise TypeError("Tuple cannot have min_items/max_items constraints")
         return json_schema(
@@ -326,12 +378,12 @@ class SchemaBuilder(SchemaVisitor[Conv, JsonSchema]):
         self._check_constraints(ObjectConstraints)
         return json_schema(
             type=JsonType.OBJECT,
-            properties={key: self.visit(cls) for key, cls in keys.items()},
+            properties={key: self.visit(tp) for key, tp in keys.items()},
             required=list(keys) if total else [],
         )
 
-    @with_schema
-    def _union_result(self, results: Sequence[JsonSchema]) -> JsonSchema:
+    def _union_result(self, results: Iterable[JsonSchema]) -> JsonSchema:
+        results = list(results)
         if len(results) == 1:
             return results[0]
         elif all(alt.keys() == {"type"} for alt in results):
@@ -358,27 +410,34 @@ class SchemaBuilder(SchemaVisitor[Conv, JsonSchema]):
         else:
             return json_schema(anyOf=results)
 
+    @with_schema
+    def union(self, alternatives: Sequence[AnyType]) -> JsonSchema:
+        return self._union_result(
+            map(self.visit, filter_skipped(alternatives, schema_only=True))
+        )
+
     def visit_with_schema(self, cls: AnyType, schema: Optional[Schema]) -> JsonSchema:
-        schema_save = self.schema
-        self.schema = schema
+        schema_save = self._schema
+        if not is_hashable(cls):
+            self._schema = schema
+        elif self.is_extra_conversions(cls):
+            self._schema = None
+        else:
+            self._schema = schema
+            ref = get_ref(cls)
+            if ref in self.refs:
+                if self._ignore_first_ref:
+                    self._ignore_first_ref = False
+                else:
+                    assert isinstance(ref, str)
+                    return self._ref_schema(ref)
+            self._merge_schema(get_schema(cls))
         try:
-            if self._is_conversion(cls):
-                self._merge_schema(get_schema(cls))
-                return self.visit_not_builtin(cls)
-            if is_hashable(cls):
-                ref = get_ref(cls)
-                if ref in self.refs:
-                    if self.ignore_first_ref:
-                        self.ignore_first_ref = False
-                    else:
-                        assert isinstance(ref, str)
-                        return self._ref_schema(ref)
-                self._merge_schema(get_schema(cls))
             return super().visit(cls)
         finally:
-            self.schema = schema_save
+            self._schema = schema_save
 
-    def visit(self, cls: AnyType):
+    def visit(self, cls: AnyType) -> JsonSchema:
         return self.visit_with_schema(cls, None)
 
     @contextmanager
@@ -393,15 +452,19 @@ class SchemaBuilder(SchemaVisitor[Conv, JsonSchema]):
     RefsExtractor: Type["RefsExtractor"]
 
 
-class DeserializationSchemaBuilder(DeserializationSchemaVisitor, SchemaBuilder):
-    class RefsExtractor(DeserializationSchemaVisitor, RefsExtractor):  # type: ignore
+class DeserializationSchemaBuilder(DeserializationVisitor, SchemaBuilder):
+    class RefsExtractor(DeserializationVisitor, RefsExtractor):  # type: ignore
         pass
 
+    visit_conversion = with_schema(DeserializationVisitor.visit_conversion)
 
-class SerializationSchemaBuilder(SerializationSchemaVisitor, SchemaBuilder):
-    class RefsExtractor(SerializationSchemaVisitor, RefsExtractor):  # type: ignore
+
+class SerializationSchemaBuilder(SerializationVisitor, SchemaBuilder):
+    class RefsExtractor(SerializationVisitor, RefsExtractor):  # type: ignore
         pass
 
+    visit_conversion = with_schema(SerializationVisitor.visit_conversion)
+
 
 TypesWithConversions = Collection[Union[AnyType, Tuple[AnyType, Conversions]]]
 
@@ -420,7 +483,7 @@ def _default_version(
     return version, ref_factory, all_refs
 
 
-def _export_refs(
+def _extract_refs(
     types: TypesWithConversions, builder: Type[SchemaBuilder], all_refs: bool
 ) -> Mapping[str, AnyType]:
     refs: Refs = {}
@@ -428,16 +491,19 @@ def _export_refs(
         conversions = None
         if isinstance(cls, tuple):
             cls, conversions = cls
-        builder.RefsExtractor(conversions, refs).visit(cls)
+        builder.RefsExtractor(refs).visit_with_conversions(cls, conversions)
     filtr = (lambda count: True) if all_refs else (lambda count: count > 1)
     return {ref: cls for ref, (cls, count) in refs.items() if filtr(count)}
 
 
 def _refs_schema(
-    builder: Type[SchemaBuilder], refs: Mapping[str, AnyType], ref_factory: RefFactory
+    builder: Type[SchemaBuilder],
+    aliaser: Aliaser,
+    refs: Mapping[str, AnyType],
+    ref_factory: RefFactory,
 ) -> Mapping[str, JsonSchema]:
     return {
-        ref: builder(None, ref_factory, refs, True).visit(cls)
+        ref: builder(aliaser, ref_factory, refs, True).visit(cls)
         for ref, cls in refs.items()
     }
 
@@ -448,6 +514,7 @@ def _schema(
     schema: Optional[Schema],
     conversions: Optional[Conversions],
     version: Optional[JsonSchemaVersion],
+    aliaser: Optional[Aliaser],
     ref_factory: Optional[RefFactory],
     all_refs: Optional[bool],
     with_schema: bool,
@@ -455,12 +522,15 @@ def _schema(
     add_defs = ref_factory is None
     if ref_factory is not None and all_refs is None:
         all_refs = True
+    if aliaser is None:
+        aliaser = settings.aliaser()
     version, ref_factory, all_refs = _default_version(version, ref_factory, all_refs)
-    refs = _export_refs([(cls, conversions)], builder, all_refs)
-    visitor = builder(conversions, ref_factory, refs, False)
-    json_schema = visitor.visit_with_schema(cls, schema)
+    refs = _extract_refs([(cls, conversions)], builder, all_refs)
+    visitor = builder(aliaser, ref_factory, refs, False)
+    with visitor._replace_conversions(conversions):
+        json_schema = visitor.visit_with_schema(cls, schema)
     if add_defs:
-        defs = _refs_schema(builder, refs, ref_factory)
+        defs = _refs_schema(builder, aliaser, refs, ref_factory)
         if defs:
             json_schema["$defs"] = defs
     result = serialize(json_schema, conversions=version.conversions)
@@ -475,6 +545,7 @@ def deserialization_schema(
     schema: Schema = None,
     conversions: Conversions = None,
     version: JsonSchemaVersion = None,
+    aliaser: Aliaser = None,
     ref_factory: RefFactory = None,
     all_refs: bool = None,
     with_schema: bool = True,
@@ -485,6 +556,7 @@ def deserialization_schema(
         schema,
         conversions,
         version,
+        aliaser,
         ref_factory,
         all_refs,
         with_schema,
@@ -497,6 +569,7 @@ def serialization_schema(
     schema: Schema = None,
     conversions: Conversions = None,
     version: JsonSchemaVersion = None,
+    aliaser: Aliaser = None,
     ref_factory: RefFactory = None,
     all_refs: bool = None,
     with_schema: bool = True,
@@ -507,6 +580,7 @@ def serialization_schema(
         schema,
         conversions,
         version,
+        aliaser,
         ref_factory,
         all_refs,
         with_schema,
@@ -516,10 +590,13 @@ def serialization_schema(
 def _defs_schema(
     types: TypesWithConversions,
     builder: Type[SchemaBuilder],
+    aliaser: Aliaser,
     ref_factory: RefFactory,
     all_refs: bool,
 ) -> Mapping[str, JsonSchema]:
-    return _refs_schema(builder, _export_refs(types, builder, all_refs), ref_factory)
+    return _refs_schema(
+        builder, aliaser, _extract_refs(types, builder, all_refs), ref_factory
+    )
 
 
 def _set_missing_properties(
@@ -536,16 +613,19 @@ def definitions_schema(
     *,
     deserialization: TypesWithConversions = (),
     serialization: TypesWithConversions = (),
+    aliaser: Aliaser = None,
     version: JsonSchemaVersion = None,
     ref_factory: Optional[RefFactory] = None,
     all_refs: bool = None,
 ) -> Mapping[str, Mapping[str, Any]]:
+    if aliaser is None:
+        aliaser = settings.aliaser()
     version, ref_factory, all_refs = _default_version(version, ref_factory, all_refs)
     deserialization_schemas = _defs_schema(
-        deserialization, DeserializationSchemaBuilder, ref_factory, all_refs
+        deserialization, DeserializationSchemaBuilder, aliaser, ref_factory, all_refs
     )
     serialization_schemas = _defs_schema(
-        serialization, SerializationSchemaBuilder, ref_factory, all_refs
+        serialization, SerializationSchemaBuilder, aliaser, ref_factory, all_refs
     )
     for duplicate in deserialization_schemas.keys() & serialization_schemas.keys():
         d_schema = deserialization_schemas[duplicate]
