@@ -11,9 +11,10 @@ from typing import (  # type: ignore
     Type,
 )
 
-from apischema.conversions.utils import Conversions
-from apischema.json_schema.generation.visitor import SchemaVisitor
+from apischema.conversions.visitor import ConversionsVisitor
+from apischema.dataclass_utils import get_field_conversion, get_fields
 from apischema.json_schema.refs import get_ref, schema_ref
+from apischema.skip import filter_skipped
 from apischema.types import AnyType
 from apischema.utils import is_hashable
 
@@ -29,13 +30,15 @@ class Recursive(Exception):
     pass
 
 
-class RefsExtractor(SchemaVisitor):
-    def __init__(self, conversions: Optional[Conversions], refs: Refs):
-        super().__init__(conversions)
+class RefsExtractor(ConversionsVisitor):
+    def __init__(self, refs: Refs):
+        super().__init__()
         self.refs = refs
 
     def _incr_ref(self, ref: Optional[str], cls: AnyType) -> bool:
-        if ref is not None:
+        if ref is None:
+            return False
+        else:
             ref_cls, count = self.refs.get(ref, (cls, 0))
             if ref_cls != cls:
                 raise ValueError(
@@ -43,7 +46,6 @@ class RefsExtractor(SchemaVisitor):
                 )
             self.refs[ref] = (ref_cls, count + 1)
             return count > 0
-        return False
 
     def annotated(self, cls: AnyType, annotations: Sequence[Any]):
         for annotation in reversed(annotations):
@@ -52,7 +54,7 @@ class RefsExtractor(SchemaVisitor):
                 ref = annotation.ref
                 if not isinstance(ref, str):
                     raise ValueError("Annotated schema_ref can only be str")
-                annotated = Annotated[(cls, *filter(is_hashable, annotations))]  # type: ignore # noqa E501
+                annotated = Annotated[(cls, *filter(is_hashable, annotations))]  # type: ignore # noqa: E501
                 if self._incr_ref(ref, annotated):
                     return
         return self.visit(cls)
@@ -70,9 +72,12 @@ class RefsExtractor(SchemaVisitor):
         fields: Sequence[Field],
         init_vars: Sequence[Field],
     ):
-
-        for field in self._dataclass_fields(cls, fields, init_vars):
-            self._visit_field(field, types[field.name])
+        for field in get_fields(fields, init_vars, self.operation):
+            field_type, conversions, _ = get_field_conversion(
+                field, types[field.name], self.operation
+            )
+            with self._replace_conversions(conversions):
+                self.visit(field_type)
 
     def enum(self, cls: Type[Enum]):
         pass
@@ -110,11 +115,19 @@ class RefsExtractor(SchemaVisitor):
         for cls in keys.values():
             self.visit(cls)
 
+    def _union_result(self, results: Iterable):
+        for _ in results:
+            pass
+
+    def union(self, alternatives: Sequence[AnyType]):
+        return self._union_result(
+            map(self.visit, filter_skipped(alternatives, schema_only=True))
+        )
+
     def visit(self, cls: AnyType):
-        if self._is_conversion(cls):
-            return self.visit_not_builtin(cls)
-        # Annotated is not always hashable
-        if is_hashable(cls):
-            if self._incr_ref(get_ref(cls), cls):
-                return
-        super().visit(cls)
+        if (
+            not is_hashable(cls)
+            or self.is_extra_conversions(cls)
+            or not self._incr_ref(get_ref(cls), cls)
+        ):
+            super().visit(cls)
