@@ -17,8 +17,8 @@ from typing import (
 from apischema.dataclass_utils import is_dataclass
 from apischema.types import AnyType
 from apischema.typing import _TypedDictMeta, get_origin
-from apischema.utils import has_free_type_vars, type_name
-from apischema.visitor import Visitor
+from apischema.utils import is_type_var, type_name
+from apischema.visitor import Unsupported, Visitor
 
 Ref = Union[str, "ellipsis", None]  # noqa: F821
 _refs: Dict[AnyType, Optional[Ref]] = {}
@@ -26,14 +26,16 @@ _refs: Dict[AnyType, Optional[Ref]] = {}
 
 def _default_ref(cls: AnyType) -> Ref:
     if not hasattr(cls, "__parameters__") and (
-        is_dataclass(cls)
+        (isinstance(cls, type) and is_dataclass(cls))
         or (hasattr(cls, "__supertype__") and is_builtin(cls))
         or isinstance(cls, _TypedDictMeta)
     ):
         return ...
     else:
         with suppress(TypeError):
-            if issubclass(cls, tuple) and hasattr(cls, "_field"):
+            if (issubclass(cls, tuple) and hasattr(cls, "_field")) or issubclass(
+                cls, Enum
+            ):
                 return ...
         return None
 
@@ -60,7 +62,9 @@ class schema_ref:
             # could be customized, but the NewType ref would then erase this
             # customization in the schema.
             raise TypeError("NewType of non-builtin type can not have a ref")
-        if has_free_type_vars(cls):
+        if is_type_var(cls):
+            raise TypeError("TypeVar cannot have a ref")
+        elif getattr(cls, "__parameters__", ()):
             raise TypeError("Unspecialized generic types cannot have a ref")
         if get_origin(cls) is not None and self.ref is ...:
             raise TypeError(f"Generic alias {cls} cannot have ... ref")
@@ -94,14 +98,11 @@ class BuiltinVisitor(Visitor):
         for alt in alternatives:
             self.visit(alt)
 
-    def unsupported(self, cls: Type):
-        raise NotImplementedError()
-
 
 def is_builtin(cls: AnyType) -> bool:
     try:
         BuiltinVisitor().visit(cls)
-    except NotImplementedError:
+    except (NotImplementedError, Unsupported):
         return False
     else:
         return True
