@@ -5,11 +5,11 @@ from apischema.utils import PREFIX
 
 ALIAS_METADATA = f"{PREFIX}alias"
 ALIAS_NO_OVERRIDE_METADATA = f"{PREFIX}alias_no_override"
-MERGED_METADATA = f"{PREFIX}merged"
 CONVERSIONS_METADATA = f"{PREFIX}conversions"
 DEFAULT_AS_SET = f"{PREFIX}default_as_set"
 DEFAULT_FALLBACK_METADATA = f"{PREFIX}default_fallback"
 INIT_VAR_METADATA = f"{PREFIX}init_var"
+MERGED_METADATA = f"{PREFIX}merged"
 POST_INIT_METADATA = f"{PREFIX}post_init"
 PROPERTIES_METADATA = f"{PREFIX}properties"
 REQUIRED_METADATA = f"{PREFIX}required"
@@ -20,7 +20,8 @@ VALIDATORS_METADATA = f"{PREFIX}validators"
 FORBIDDEN_WITH_AGGREGATE = {
     ALIAS_METADATA,
     ALIAS_NO_OVERRIDE_METADATA,
-    CONVERSIONS_METADATA,
+    DEFAULT_AS_SET,
+    POST_INIT_METADATA,
     REQUIRED_METADATA,
     SCHEMA_METADATA,
     VALIDATORS_METADATA,
