@@ -1,15 +1,9 @@
 """Kind of typing_extensions for this package"""
-__all__ = [
-    "_LiteralMeta",
-    "_TypedDictMeta",
-    "get_type_hints",
-    "get_origin",
-    "get_args",
-]
+__all__ = ["_LiteralMeta", "_TypedDictMeta", "get_args", "get_origin", "get_type_hints"]
 
 import sys
 from types import ModuleType
-from typing import Any, Callable, Generic, TypeVar
+from typing import Any, Callable, Generic, Tuple, TypeVar
 
 
 class _FakeType:
@@ -48,7 +42,20 @@ else:  # pragma: no cover
         from typing_extensions import get_origin, get_args
     except ImportError:
 
+        def _assemble_tree(tree: Tuple[Any]) -> Any:
+            if not isinstance(tree, tuple):
+                return tree
+            else:
+                origin, *args = tree  # type: ignore
+                if origin is Annotated:
+                    return Annotated[(_assemble_tree(args[0]), *args[1])]
+                else:
+                    return origin[tuple(map(_assemble_tree, args))]
+
         def get_origin(tp):  # type: ignore
+            # In Python 3.6: List[Collection[T]][int]._args__ == int != Collection[int]
+            if hasattr(tp, "_subs_tree"):
+                tp = _assemble_tree(tp._subs_tree())
             if isinstance(tp, _AnnotatedAlias):
                 return Annotated
             if tp is Generic:
@@ -56,6 +63,9 @@ else:  # pragma: no cover
             return getattr(tp, "__origin__", None)
 
         def get_args(tp):  # type: ignore
+            # In Python 3.6: List[Collection[T]][int]._args__ == int != Collection[int]
+            if hasattr(tp, "_subs_tree"):
+                tp = _assemble_tree(tp._subs_tree())
             if isinstance(tp, _AnnotatedAlias):
                 return (tp.__args__[0], *tp.__metadata__)
             res = getattr(tp, "__args__", ())
@@ -65,10 +75,10 @@ else:  # pragma: no cover
 
 
 if sys.version_info >= (3, 8):  # pragma: no cover
-    from typing import Literal, TypedDict, Protocol  # noqa F401
+    from typing import Literal, TypedDict, Protocol  # noqa: F401
 else:  # pragma: no cover
     try:
-        from typing_extensions import Literal, TypedDict, Protocol  # noqa F401
+        from typing_extensions import Literal, TypedDict, Protocol  # noqa: F401
     except ImportError:
         pass
 
